(function () {
  function hasClass(elem, classes) {
    if (typeof classes === 'string') {
      classes = [classes];
    }
    var elem_classes = elem.className.split(' ');

    for (var c = 0; c < classes.length; c++) {
      if (elem_classes.indexOf(classes[c]) < 0) {
        return false;
      }
    }

    return true;
  }

  function forChildren(elem, func) {
    if (elem && elem.children && elem.children.length) {
      for (var c = 0; c < elem.children.length; c++) {
        if (elem.children[c] !== elem) {
          func(elem.children[c], c, elem.children);
        }
      }
    }
  }

  function forSiblings(elem, func) {
    if (elem) {
      forChildren(elem.parentElement, func);
    }
  }

  function getHeirarchy(elem) {
    var heirarchy = [];

    while (elem) {
      heirarchy.push(elem);
      elem = elem.parentElement;
    }

    return heirarchy;
  }

  function activateMenu(elem) {
    var touched = [], close = [].slice.call(document.querySelectorAll('.dropdown-menus .menu, .dropdown-menus .menu-item, .dropdown-menus .menu-link')), count = 0;

    getHeirarchy(elem).forEach(elem => {
      elem.classList.add('open');
      touched.push(elem);
      count++;

      if (hasClass(elem, 'menu')) {
        elem.setAttribute('aria-hidden', false);

        forSiblings(elem, function (sib) {
          touched.push(sib);
          count++;
          if (hasClass(sib, 'menu-link')) {
            sib.classList.add('open');
          }
        });
      } else if (hasClass(elem, 'menu-link')) {
        forSiblings(elem, function (sib) {
          touched.push(sib);
          count++;
          if (hasClass(sib, 'menu')) {
            sib.classList.remove('open');
            sib.setAttribute('aria-hidden', false);
          }
        });
      }
    });

    close = close.filter(elem => touched.indexOf(elem) < 0);

    close.forEach(elem => {
      elem.classList.remove('open');

      if (hasClass(elem, 'menu')) {
        elem.setAttribute('aria-hidden', true);
      }
    });

    return touched.length;
  }

  function clickHandler (e) {
    activateMenu(e.target);

    if (hasClass(e.target, ['menu-link', 'has-submenu', 'menu-handler'])) {
      e.preventDefault();
      e.stopPropagation();
    }
  }

  function keyupHandler (e) {
    if (e.keyCode === 27 && activateMenu()) {
      e.preventDefault();
      e.stopPropagation();
    }
  }

  Drupal.behaviors.soahMenus = {
    attach: function () {
      var elems = document.querySelectorAll('.dropdown-menus .menu-link.has-submenu');

      for (var c = 0; c < elems.length; c++) {
        elems[c].classList.add('menu-handler');
        elems[c].setAttribute('aria-haspopup', true);

        forSiblings(elems[c], function (sib) {
          if (hasClass(sib, 'menu') && !sib.getAttribute('aria-hidden')) {
            sib.setAttribute('aria-hidden', true);
          }
        });
      }

    }
  };

  document.addEventListener('click', clickHandler);
  document.addEventListener('keyup', keyupHandler);
  Drupal.behaviors.soahMenus.attach();
})();
